/*
 * Copyright (C) 2012  Christian Mollekopf <mollekopf@kolabsys.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TESTHELPERS_H
#define TESTHELPERS_H

// #include <contact.h>

#include <QtCore/QObject>
#include <QtTest/QtTest>
#include <kcalcore/recurrence.h>
#include <kcalcore/event.h>
#include <kcalcore/todo.h>
#include <kcalcore/journal.h>

#include <kolabformat.h>

Q_DECLARE_METATYPE(Kolab::Duration);
Q_DECLARE_METATYPE(Kolab::cDateTime);
Q_DECLARE_METATYPE(std::vector<Kolab::cDateTime>);
Q_DECLARE_METATYPE(Kolab::Event);
Q_DECLARE_METATYPE(std::vector<Kolab::Event>);
Q_DECLARE_METATYPE(Kolab::Todo);
Q_DECLARE_METATYPE(Kolab::Journal);
Q_DECLARE_METATYPE(Kolab::Contact);
Q_DECLARE_METATYPE(Kolab::Period);
Q_DECLARE_METATYPE(std::vector<Kolab::FreebusyPeriod>);

Q_DECLARE_METATYPE(KCalCore::Event);
Q_DECLARE_METATYPE(KCalCore::Todo);
Q_DECLARE_METATYPE(KCalCore::Journal);

namespace {

    // Appends a C string to a QByteArray and delete[]s the string argument.
    // Returns a reference to the QByteArray.
    QByteArray &append(QByteArray &ba, const char *string)
    {
        ba += QByteArray(string);
        delete[] string;
        return ba;
    }

}

namespace QTest {
    
    template<>
    char *toString(const Kolab::cDateTime &dt)
    {
        QByteArray ba = "Kolab::cDateTime(";
        ba += QByteArray::number(dt.year()) + ", " + QByteArray::number(dt.month())+ ", " + QByteArray::number(dt.day()) + ", ";
        ba += QByteArray::number(dt.hour()) + ", " + QByteArray::number(dt.minute()) + ", " + QByteArray::number(dt.second())+ ", ";
        ba += QByteArray(dt.isUTC()?QByteArray("UTC"):QByteArray("TZ: "+QByteArray::fromStdString(dt.timezone())));
        ba += ")";
        return qstrdup(ba.data());
    }
    
    // template<>
    // char *toString(const QDateTime &dt)
    // {
    //     QByteArray ba = "QDateTime(";
    //     ba += dt.toString().toAscii();
    //     ba += dt.timeZone().name().toAscii();
    //     ba += ")";
    //     return qstrdup(ba.data());
    // }

    template<>
    char *toString(const KCalCore::Attendee &at)
    {
        QByteArray ba = "Attendee(";
        ba += at.name().toLatin1() + ", ";
        ba += at.email().toLatin1() + ", ";
        ba += QByteArray::number(at.role()) + ", ";
        ba += QByteArray::number(at.status()) + ", ";
        ba += QByteArray::number(at.RSVP()) + ", ";
        ba += at.delegate().toLatin1() + ", ";
        ba += at.delegator().toLatin1() + ", ";
        ba += at.uid().toLatin1() + ", ";
        ba += ")";
        return qstrdup(ba.data());
    }


    template<>
    char *toString(const QList<int> &l)
    {
        QByteArray ba = "QList<int>(";
        foreach(int i, l) {
            ba += QByteArray::number(i) + ", ";
        }
        ba += ")";
        return qstrdup(ba.data());
    }

    template<>
    char *toString(const QList<KCalCore::RecurrenceRule::WDayPos> &l)
    {
        QByteArray ba = "QList<int>(";
        foreach(const KCalCore::RecurrenceRule::WDayPos &i, l) {
            ba += QByteArray::number(i.pos()) + " ";
            ba += QByteArray::number(i.day()) + ", ";
        }
        ba += ")";
        return qstrdup(ba.data());
    }

    template<>
    char *toString(const KCalCore::DateList &l)
    {
        QByteArray ba = "KCalCore::DateList(";
        foreach(const QDate &i, l) {
            ba += i.toString().toLatin1();
        }
        ba += ")";
        return qstrdup(ba.data());
    }

    template<>
    char *toString(const KCalCore::DateTimeList &l)
    {
        QByteArray ba = "KCalCore::DateTimeList(";
        foreach(const QDateTime &i, l) {
            append(ba, toString(i));
        }
        ba += ")";
        return qstrdup(ba.data());
    }

    template<>
    char *toString(const KCalCore::Recurrence &at)
    {
        // at.dump();
        KCalCore::RecurrenceRule *r = at.defaultRRule();
        QByteArray ba;
        if (!r) {
            ba += "Recurrence( )";
        } else {
            Q_ASSERT(r);
            Q_ASSERT(at.rRules().size() == 1);

            ba += "Recurrence(";
            ba += QByteArray::number(r->recurrenceType()) + "\n";
            ba += QByteArray::number(r->frequency()) + "\n";
            ba += QByteArray::number(r->duration()) + "\n";
            append(ba, toString(r->startDt())) += "\n";
            append(ba, toString(r->endDt())) += "\n";
            append(ba, toString(r->bySeconds())) += "\n";
            append(ba, toString(r->byMinutes())) += "\n";
            append(ba, toString(r->byHours())) += "\n";
            append(ba, toString(r->byDays())) += "\n";
            append(ba, toString(r->byMonthDays())) += "\n";
            append(ba, toString(r->byYearDays())) += "\n";
            append(ba, toString(r->byMonths())) += "\n";
            ba += ")\n";
            append(ba, toString(at.exDates())) += "\n";
            append(ba, toString(at.exDateTimes())) += "\n";
            append(ba, toString(at.rDates())) += "\n";
            append(ba, toString(at.rDateTimes())) += "\n";

        }
        return qstrdup(ba.data());
    }

    template<>
    char *toString(const Kolab::RecurrenceRule &at)
    {
        QByteArray ba;
        ba += "KolabRecurrenceRule(";
        ba += QByteArray::number(at.weekStart()) + "\n";
        ba += QByteArray::number(at.frequency()) + "\n";
        ba += QByteArray::number(at.interval()) + "\n";
        ba += QByteArray::number(at.count()) + "\n";
        append(ba, toString(at.end())) += "\n";
        append(ba, toString(at.bysecond())) += "\n";
        append(ba, toString(at.byminute())) += "\n";
        append(ba, toString(at.byhour())) += "\n";
        append(ba, toString(at.byday())) += "\n";
        append(ba, toString(at.bymonthday())) += "\n";
        append(ba, toString(at.byyearday())) += "\n";
        append(ba, toString(at.byweekno())) += "\n";
        append(ba, toString(at.bymonth())) += "\n";
        ba += ")";
        return qstrdup(ba.data());
    }

    template<>
    char *toString(const KCalCore::Duration &d)
    {
        QByteArray ba;
        ba += "KCalCore::Duration(";
        ba += QByteArray::number(d.isDaily()) + ", ";
        ba += QByteArray::number(d.value()) + " ";
        ba += ")";
        return qstrdup(ba.data());
    }

    template<>
    char *toString(const Kolab::ContactReference &a)
    {
        QByteArray ba = "Kolab::ContactReference(";
        ba += QByteArray::fromStdString(a.email()) + ", ";
        ba += QByteArray::fromStdString(a.name()) + ", ";
        ba += QByteArray::fromStdString(a.uid());
        ba += ")";
        return qstrdup(ba.data());
    }

    template<>
    char *toString(const std::vector<Kolab::ContactReference> &v)
    {
        QByteArray ba = "vector<Kolab::ContactReference>(";
        for (std::size_t i = 0; i < v.size(); i++) {
                append(ba, toString(v.at(i))) += "\n";
            }
            ba += ")";
            return qstrdup(ba.data());
    }

    template<>
    char *toString(const Kolab::Attendee &a)
    {
        QByteArray ba = "Kolab::Attendee(";
        ba += QByteArray::fromStdString(a.contact().email()) + "\n";
        ba += QByteArray::fromStdString(a.contact().name()) + "\n";
        ba += QByteArray::number(a.partStat()) + "\n";
        ba += QByteArray::number(a.role())  + "\n";
        ba += QByteArray::number(a.rsvp())  + "\n";
        ba += QByteArray::fromStdString(a.contact().uid())+"\n";
        append(ba, toString(a.delegatedTo())) += "\n";
        append(ba, toString(a.delegatedFrom())) += "\n";
        ba += QByteArray::number(a.cutype())+ "\n";
        ba += ")";
        return qstrdup(ba.data());
    }

    template<>
    char *toString(const std::vector<Kolab::Attendee> &v)
    {
        QByteArray ba = "vector<Kolab::Attendee>(";
        for (std::size_t i = 0; i < v.size(); i++) {
            append(ba, toString(v.at(i))) += "\n";
            ba += QByteArray("#######################")+ "\n";
        }
        ba += ")";
        return qstrdup(ba.data());
    }

    template<>
    char *toString(const Kolab::CustomProperty &a)
    {
        QByteArray ba = "Kolab::CustomProperty(";
        ba += QByteArray::fromStdString(a.identifier) + ", ";
        ba += QByteArray::fromStdString(a.value);
        ba += ")";
        return qstrdup(ba.data());
    }

    template<>
    char *toString(const std::vector<Kolab::CustomProperty> &v)
    {
        QByteArray ba = "vector<Kolab::CustomProperty>(";
        for (std::size_t i = 0; i < v.size(); i++) {
            append(ba, toString(v.at(i))) += "\n";
        }
        ba += ")";
        return qstrdup(ba.data());
    }

    template<>
    char *toString(const Kolab::Period &p)
    {
        QByteArray ba = "Kolab::Period(";
        append(ba, toString(p.start)) += "\n";
        append(ba, toString(p.end)) += "\n";
        ba += ")";
        return qstrdup(ba.data());
    }

    template<>
    char *toString(const std::vector<Kolab::Period> &v)
    {
        QByteArray ba = "vector<Kolab::Period>(";
        for (std::size_t i = 0; i < v.size(); i++) {
            append(ba, toString(v.at(i))) += "\n";
        }
        ba += ")";
        return qstrdup(ba.data());
    }

    template<>
    char *toString(const Kolab::FreebusyPeriod &p)
    {
        QByteArray ba = "Kolab::FreebusyPeriod(";
        ba += QByteArray::number(p.type())+ "\n";
        ba += QByteArray::fromStdString(p.eventUid())+ "\n";
        ba += QByteArray::fromStdString(p.eventLocation())+ "\n";
        ba += QByteArray::fromStdString(p.eventSummary())+ "\n";
        append(ba, toString(p.periods())) += "\n";
        ba += ")";
        return qstrdup(ba.data());
    }

    template<>
    char *toString(const Kolab::Duration &p)
    {
        QByteArray ba = "Kolab::Duration";
        ba += p.isNegative() ? "-": "+";
        ba += "(";
        ba += QByteArray::number(p.weeks())+ ", ";
        ba += QByteArray::number(p.days())+ ", ";
        ba += QByteArray::number(p.hours())+ ", ";
        ba += QByteArray::number(p.minutes())+ ", ";
        ba += QByteArray::number(p.seconds());
        ba += ")";
        return qstrdup(ba.data());
    }

}

#endif
